from melee import enums

"""Get the X coordinate of the edge of the stage, approaching from off_stage
IE: This is your X coordinate when hanging on the edge
NOTE: The left edge is always the same, but negative"""
def edgeposition(stage):
    if stage == enums.Stage.BATTLEFIELD:
        return 71.3078536987
    if stage == enums.Stage.FINAL_DESTINATION:
        return 88.4735488892
    if stage == enums.Stage.DREAMLAND:
        return 80.1791534424
    if stage == enums.Stage.FOUNTAIN_OF_DREAMS:
        return 66.2554016113
    if stage == enums.Stage.POKEMON_STADIUM:
        return 90.657852
    if stage == enums.Stage.YOSHI_STORY:
        return 58.907848

    #TODO: Other stages? Do we care?
    return 1000;

"""Get the X coordinate of the edge of the stage, while standing on the stage
IE: This is your X coordinate when teetering on the edge
NOTE: The left edge is always the same, but negative"""
def edgegroundposition(stage):
    if stage == enums.Stage.BATTLEFIELD:
        return 68.4000015259
    if stage == enums.Stage.FINAL_DESTINATION:
        return 85.5656967163
    if stage == enums.Stage.DREAMLAND:
        return 77.2713012695
    if stage == enums.Stage.FOUNTAIN_OF_DREAMS:
        return 63.3475494385
    if stage == enums.Stage.POKEMON_STADIUM:
        return 87.75
    if stage == enums.Stage.YOSHIS_STORY:
        return 56

    #TODO: Other stages? Do we care?
    return 1000;



import csv
import time
import datetime
import os

class Logger:
    def __init__(self):
        timestamp = datetime.datetime.fromtimestamp(time.time())
        #Create the Pipes directory if it doesn't already exist
        if not os.path.exists("Logs/"):
            os.makedirs("Logs/")
        self.csvfile = open('Logs/' + str(timestamp) + '.csv', 'w')
        fieldnames = ['Frame', 'Opponent x',
            'Opponent y', 'AI x', 'AI y', 'Opponent Facing', 'AI Facing',
            'Opponent Action', 'AI Action', 'Opponent Action Frame', 'AI Action Frame',
            'Opponent Jumps Left', 'AI Jumps Left', 'Opponent Stock', 'AI Stock',
            'Opponent Percent', 'AI Percent', 'Buttons Pressed', 'Notes', 'Frame Process Time']
        self.writer = csv.DictWriter(self.csvfile, fieldnames=fieldnames, extrasaction='ignore')
        self.current_row = dict()
        self.rows = []
        self.filename = self.csvfile.name

    def log(self, column, contents, concat=False):
        #Should subsequent logs be cumulative?
        if concat:
            if column in self.current_row:
                self.current_row[column] += contents
            else:
                self.current_row[column] = contents
        else:
            self.current_row[column] = contents

    #Log any common per-frame items
    def logframe(self, gamestate):
        ai_state = gamestate.ai_state
        opponent_state = gamestate.opponent_state

        self.log('Frame', gamestate.frame)
        self.log('Opponent x', str(opponent_state.x))
        self.log('Opponent y', str(opponent_state.y))
        self.log('AI x', str(ai_state.x))
        self.log('AI y', str(ai_state.y))
        self.log('Opponent Facing', str(opponent_state.facing))
        self.log('AI Facing', str(ai_state.facing))
        self.log('Opponent Action', str(opponent_state.action))
        self.log('AI Action', str(ai_state.action))
        self.log('Opponent Action Frame', str(opponent_state.action_frame))
        self.log('AI Action Frame', str(ai_state.action_frame))
        self.log('Opponent Jumps Left', str(opponent_state.jumps_left))
        self.log('AI Jumps Left', str(ai_state.jumps_left))
        self.log('Opponent Stock', str(opponent_state.stock))
        self.log('AI Stock', str(ai_state.stock))
        self.log('Opponent Percent', str(opponent_state.percent))
        self.log('AI Percent', str(ai_state.percent))

    def writeframe(self):
        self.rows.append(self.current_row)
        self.current_row = dict()

    def writelog(self):
        self.writer.writeheader()
        self.writer.writerows(self.rows)



"""Helper functions for with some techskill examples"""
from melee import enums

"""Frame-perfect Multishines as Fox"""
def multishine(ai_state, controller):
    #If standing, shine
    if ai_state.action == enums.Action.STANDING:
        controller.press_button(enums.Button.BUTTON_B)
        controller.tilt_analog(enums.Button.BUTTON_MAIN, .5, 0)
        return

    #Shine on frame 3 of knee bend, else nothing
    if ai_state.action == enums.Action.KNEE_BEND:
        if ai_state.action_frame == 3:
            controller.press_button(enums.Button.BUTTON_B)
            controller.tilt_analog(enums.Button.BUTTON_MAIN, .5, 0)
            return
        else:
            controller.empty_input()
            return

    isInShineStart = (ai_state.action == enums.Action.DOWN_B_STUN or
            ai_state.action == enums.Action.DOWN_B_GROUND_START)

    #Jump out of shine
    if isInShineStart and ai_state.action_frame >= 4 and ai_state.on_ground:
        controller.press_button(enums.Button.BUTTON_Y)
        return

    if ai_state.action == enums.Action.DOWN_B_GROUND:
        controller.press_button(enums.Button.BUTTON_Y)
        return

    controller.empty_input()

"""Spam upsmashes"""
def upsmashes(ai_state, controller):
    if ai_state.action == enums.Action.STANDING:
        controller.tilt_analog(enums.Button.BUTTON_C, .5, 1)
        return

    controller.empty_input()
